/*
 *  Shuangge Ma.  Copyright (c) 2002 UW-Madison
 */
#include "TrialEndNormal.h"
#include "TrialEndSurvival.h"

#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>

// OPENGL INCLUDES
//#include <GL/Glos.h>
#include <GL/glaux.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h> 

#define TRUE 1
#define FALSE 0

#define PI			3.1415927
#define Pi			3.1415926
#define MAXLOOK		10

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

#define GAP			10
#define MainWidth	1000
#define MainHeight	700
#define UpperHeight	550
#define LowerHeight	150

#define Y_GAP		40
#define Y_Line1		60
#define Y_Line2		100
#define Y_Line3		140
#define Y_Line4		180
#define Y_Line5		220
#define Y_Line6		260
#define Y_Line7		300
#define Y_Line8		340
#define Y_Line9		380
#define Y_Line10	420
#define Y_Line11	460
#define Y_Line12	500
#define Y_Line13	540
#define Y_Line14	580
#define Y_Line15	620
#define Y_Line16	660

#define X_Column0	25
#define X_Column1	50
#define X_Column2	75
#define X_Column3	100
#define X_Column4	125
#define X_Column10	500
#define X_Column15  750
#define X_Column_Mean	450
#define X_Column_Var	600


// ftoa: convert a floating point number to ascii.
// itoa: convert a integer number to ascii.      
//
void ftoa(float i);
void itoa(int i);
void ftoa_2(float f);
void ftoa_3(float f);
void ftoa_4(float f);
double *Stand_Normal(int i);   
double* sqrt_matrix(int column, double *posimatrix );

	
//
// readfile: loads a data profile, put the data into array targetName,
//
int readfile(char *fileName, char *targetName);
//
// clearString: clears the contents of a string
//
void clearString(char * string);
char * trimString(char* string);

//
// bitmap_output: Function to draw characters in
//        opengl using the glut toolkit
//
void bitmap_output(int x, int y, char *string, void *font);

///////////////////////////////////////////////////////////////////////////////
//																			 //
//                         Broadcast Window Functions						 //
//																			 //
///////////////////////////////////////////////////////////////////////////////
//
// displayBroadcast: display callback for the graphics
//        window.
void displayBroadcast(void);

// BroadcastKey: Function to handle keyboard events for input
//        in to the program via the broadcast window.
//
void BroadcastKey(unsigned char key, int x, int y);

///////////////////////////////////////////////////////////////////////////////
//																			 //
//                         Controls Window Functions						 //
//																			 //
///////////////////////////////////////////////////////////////////////////////

//
// displayControls: display callback to manage the display 
//        of controls window
//
void displayControls(void);


//
// ControalMouse: function to determine which choice is clicked,
//      and what to do about it.
//
void ControlMouse(int b, int s, int x, int y);

//
//	ControlMousePassive: Passive window function to determine 
//      when the mouse passes over a selection choice.  If so, the 
//      choice is highlighted.
// 
void ControlMousePassive(int x, int y);

///////////////////////////////////////////////////////////////////////////////
//																			 //
//                         Credits Window Functions							 //
//																			 //
///////////////////////////////////////////////////////////////////////////////


//
// displayCredits: Function to handle the display of the credits
//            window presented when the program is started.
//
void displayCredits(void);

//
// reshapeCredits: This is needed because the credits window is a 
//          root level window.
// 
void reshapeCredits(int w, int h);

//
// CreditMouse: Function to handle the click to begin the program
//
void CreditMouse(int b, int s, int x, int y);


///////////////////////////////////////////////////////////////////////////////
//																			 //
//                         Shared Window Functions							 //
//																			 //
///////////////////////////////////////////////////////////////////////////////


//
// Generic display callback 
//
void display(void);
//
// Generic IsVisable callback
//
void vis(int visState);

//
// Generic IsVisable callback
//
void visCredit(int visState);

//
// General reshape function to handle creation
// and managment of the root interface window.
//
void reshape(int w, int h);

//
// Function to initialize and pop the main
// interface window.
//
void BuildInterface(void);

//
// Exit function for the 3D and credit winodows.
//
void ExitKey(unsigned char key, int x, int y);


/////////////////////////
// Math function
//
void getSquare(double x);




/////////////////////
void Uni_endpoint( );
//the main function for the Uni-endpoint case


double absD(double);
// absolute value

int minInt(int x, int y);


int LoadDataN1(char*, int);
int LoadDataN2(char*, int);
int LoadDataS1(char*);
int LoadDataS2(char*);


//
//	The Uni Variate Normal Case
//
void UniNormalDecision();

	int *UNSize1();
	int *UNSize2();
	// find the sample size for each look

	double *UNMean(TrialEndNormal *);

	double *UNVar(TrialEndNormal *);

	double *UNCov(TrialEndNormal *);

	double *UNMeanNew(TrialEndNormal *);

	double *UNVarNew(TrialEndNormal *);

	double *UNCovNew(TrialEndNormal *);

	double *UNParameterEstimate();
		// The estimate of mean, var and cov function.
	
	double *UNNormalSimul(int , int , double *, double *, double *);
		// Simulate according to previous structure.

	double *UNBound(int, double *, double *, TrialEndNormal *);
	// The boundary calculation:


//
//	The Two Normal Case:
//
void TwoNormalDecision();

	double *TNMean(TrialEndNormal *);

	double *TNVar(TrialEndNormal *);

	double *TNCov(TrialEndNormal *);

	double *TNMeanNew(TrialEndNormal *);

	double *TNVarNew(TrialEndNormal *);

	double *TNCovNew(TrialEndNormal *);


	double *TNCrossCov();

	double *TNCrossCovNew();

	void TNInitialize(TrialEndNormal *);

	double *TNUniBound(double *, double , int, int , int ,int);

	double *TNZSimul(double *, int , int , int );

	double *TNZSimulA(double *, int , int , int );


//
//	The Uni Survival Case:
//
void UniSurvivalDecision();

//
//	The Two survival case
//
void TwoSurvivalDecision();

	double * LogRank2(int , int , int, int, int , double *, double *, double *, double *, double*);
	
	// the latest version of logrank test statistics, made by shuangge Ma 11/23/02
	double * LogRank3(int , int , int, int, int , double *, double *, double *, double *, double*);
	
	double * TSCov(int , int , int, int, int , double *, double *, double *, double *, double*);

	double *TSSort1(int, int, double *, double *);

	double *SortTSCov(int, int, double *, double *, double *);


//
//	Normal Survival Case:
//
void NormalSurvivalDecision();

	double *SortNSCov(int, int, double *, double *, double *, double *);
		// mainly copied from SortTSCov. 
		// include the indentify array
	double *NSSort1(int , int , double *, double *, double *);


//
//	Survival Normal Case
//	
void SurvivalNormalDecision();



//
//	Ancillary Functions:
//

double *alpha_beta1(double, double *, int);

double *alpha_beta2(double, double *, int);

double *alpha_beta3(double, double *, int);

double absd(double);

double Sample_Quan(int, double*, double);

float ran1(long *idum);

double *NormalCheck(int, int, double *);



void OutNormal(TrialEndNormal *);


void mySort(double *, int );